#!/bin/bash
VER=1.2

#--[ Info ]--------------------------------------------------------------#
#                                                                        #
# A script to verify the integrity between users/passwd/group files.     #
# It will start with verifying that all entries in the passwd has an     #
# userfile.                                                              #
# It will then check that all users has an entry in the passwd file.     #
# After that, it will check that the primary group in the userfile is    #
# the one specified in the passwd file that user.                        #
#                                                                        #
# V1.2+ will NOT touch your userfiles or passwd file for safety.         #
# It will create a new passwd file which you can overwrite with your old #
# one, once you verified it.                                             #
#                                                                        #
# Just change the settings below and chmod this script to 700 or so. Run #
# it as root.                                                            #
#                                                                        #
# 1.2  : Added passwd/users verification first. Inconsistencies here     #
#        could lead to an empty passwd file.                             #
#                                                                        #
#        No longer overwrites the current passwd file. Instead, it makes #
#        a new one. Specify where with NEW_PASSWD=                       #
#                                                                        #
#        'fix' argument is no longer needed. A new file will be created  #
#        if needed.                                                      #
#                                                                        #
#        If a user has no group in the userfile, it will check that its  #
#        gid 100 in the passwd file. Previously, the user was skipped.   #
#                                                                        #
#        Thanks to CyberScream for the info based on users in no groups. #
#        Thanks to Maxpecks for getting an empty passwd file and testing #
#        1.2 with me. =)                                                 #
#                                                                        #
# 1.1  : passwd file is now resorted after being fixed.                  #
#        Run argument 'resort' to force a resort (of a previously        #
#        fixed passwd file).                                             #
#        This is a must or the user can not ever be purged incase he is  #
#        'fixed' with this script. Thanks CyberScream for the info.      #
#                                                                        #
#-[ Settings ]-----------------------------------------------------------#

## Path to users dir. Wont be touched
USERS=/glftpd/ftp-data/users

## Path to passwd file. Wont be touched
PASSWD=/glftpd/etc/passwd

## Where to create the new passwd file. Will be touched =)
NEW_PASSWD=/glftpd/etc/passwd.new


#--[ Script Start ]---------------------------------------#

if [ ! -d "$USERS" ]; then
  echo "Error. USERS dir does not exist or isnt a dir: $USERS"
  exit 0
fi

if [ ! -e "$PASSWD" ]; then
  echo "Error. passwd does not exist in defined path: $PASSWD"
  exit 0
else
  groupfile=`dirname $PASSWD`
  if [ ! -e "$groupfile/group" ]; then
    echo "Error. group file not found. Should be in same dir as the passwd, ie $groupfile"
    exit 0
  fi

  cp -f "$PASSWD" "$NEW_PASSWD"
  if [ ! -e "$NEW_PASSWD" ]; then
    echo "Error. Couldnt copy $PASSWD to $NEW_PASSWD for some reason. Quitting."
    exit 1
  fi
fi

echo "--[ Checking that all users in passwd exists as userfiles ]--"
for line in `cat $NEW_PASSWD | cut -d ':' -f1`; do
  if [ ! -e $USERS/$line ]; then
    echo "$line has a passwd entry, but no userfile."
    foundone="YES"
    errors="TRUE"
  fi
done
if [ "$foundone" != "YES" ]; then
  echo "All users in passwd file has a userfile"
fi

echo "--[ Checking that all user files exist in the passwd ]--"

unset foundone
cd $USERS

for user in `grep "^FLAGS " * | cut -d ':' -f1 | grep -v "default.user" `; do
  if [ -z "`cat "$NEW_PASSWD" | grep "^$user\:"`" ]; then
    echo "$user has a userfile, but no passwd entry!"
    foundone="YES"
    errors="TRUE"
  fi
done
if [ "$foundone" != "YES" ]; then
  echo "All userfiles match a user in passwd file"
  echo ""
fi

if [ "$errors" = "TRUE" ]; then
  echo "I will not check the passwd GID's until the above errors are sorted. Quitting now."
  exit 0
fi

proc_resort() {
  cat $NEW_PASSWD | tr ':' ' '| sort -k3,3 -n | tr ' ' ':' > /tmp/passwd.tmp
  if [ ! -e "/tmp/passwd.tmp" ]; then
    echo "Error. Can not find the newly resorted passwd file in /tmp/passwd.tmp"
    exit 1
  else
    if [ -z "`grep "." /tmp/passwd.tmp`" ]; then
      echo "Error. Cant read anything from /tmp/passwd.tmp. Did it get created ok?"
      exit 1
    fi
  fi

  cp -f /tmp/passwd.tmp $NEW_PASSWD
  rm -f /tmp/passwd.tmp

  echo "passwd file resorted ok."
}

if [ "$1" = "resort" ]; then
  proc_resort
  exit 0
fi

proc_fixnogroup() {
  if [ "`grep "^$user:" $NEW_PASSWD | cut -d ':' -f4`" != "100" ]; then
    echo "$user has no primary group in groupfile. GID in passwd isnt 100. Fixing."

    proc_getvalues

    echo "Old: $rawdata"
    echo "New: $user:$pass:$uid:100:$dateadded:$homedir:$crap"

    grep -v "$rawdata" $NEW_PASSWD > /tmp/passwd.tmp
    cp -f /tmp/passwd.tmp $NEW_PASSWD
    echo "$user:$pass:$uid:100:$dateadded:$homedir:$crap" >> $NEW_PASSWD
  fi
}

proc_getvalues() {
  rawdata=`grep "^$user:" $NEW_PASSWD`
  pass=`echo "$rawdata" | cut -d ':' -f2`
  uid=`echo "$rawdata" | cut -d ':' -f3`
  pregid=`echo "$rawdata" | cut -d ':' -f4`
  dateadded=`echo "$rawdata" | cut -d ':' -f5`
  homedir=`echo "$rawdata" | cut -d ':' -f6`
  crap=`echo "$rawdata" | cut -d ':' -f7`
  echo ""
}

echo "--[ Starting GID check on users/passwd/group files ]--"

for user in `grep "^FLAGS " * | cut -d ':' -f1 | grep -v "default.user" `; do
  groupnow=`grep "^GROUP " $user | head -n1 | cut -d ' ' -f2`
  if [ "$groupnow" ]; then
    gidinpasswd=`grep "^$user:" $NEW_PASSWD | cut -d ':' -f4`
    gidingroup=`grep "^$groupnow:" $groupfile/group | cut -d ':' -f3`

    if [ "$gidinpasswd" != "$gidingroup" ]; then
      echo "$user: $groupnow has >$gidingroup< in group file but $user has >$gidinpasswd< in passwd"

      proc_getvalues
      echo "Old: $rawdata"
      echo "New: $user:$pass:$uid:$gidingroup:$dateadded:$homedir:$crap"

      grep -v "$rawdata" $NEW_PASSWD > /tmp/passwd.tmp
      cp -f /tmp/passwd.tmp $NEW_PASSWD
      echo "$user:$pass:$uid:$gidingroup:$dateadded:$homedir:$crap" >> $NEW_PASSWD
    fi
  else
    proc_fixnogroup
  fi
done

if [ -z "$rawdata" ]; then
  echo "All users are ok!"
  rm -f "$NEW_PASSWD"
  exit 0
else
  proc_resort
  echo "A new passwd created in $NEW_PASSWD"
  exit 0
fi
